{% extends "django-dans-emails/base.html" %}

{% block content %}
<tr>
	{% if team_name %}
		<td>This is an email from {{ team_name }}.</td>
	{% else %}
		<td>This is an email.</td>
	{% endif %}
</tr>
<tr>
	<td><br/></td>
</tr>
<tr>
	<td>Have a great day!</td>
</tr>
{% endblock %}