
{% extends "django-dans-emails/base.html" %}

{% block content %}
	<tr>
		<td>You've successfully requested a password reset change. Please visit this url to update it:</td>
	</tr>
	<tr>
		<td><a href="{{ url_password_reset }}">Password Reset Request</a></td>
	</tr>
	<tr>
		<td><br/></td>
	</tr>
	<tr>
		<td>Hope you enjoy using our site and have a great day!</td>
	</tr>
{% endblock %}